from __future__ import division

from enigma import eServiceReference, eConsoleAppContainer
from Components.MediaPlayer import PlayList
from Plugins.Plugin import PluginDescriptor
from Components.ActionMap import ActionMap
from Components.Label import Label
from Components.Button import Button
from Components.GUIComponent import GUIComponent
from enigma import eListboxPythonMultiContent, eListbox, gFont, \
	RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_VALIGN_CENTER
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Components.config import config, ConfigSubsection, ConfigInteger, ConfigYesNo, ConfigText, getConfigListEntry
from Components.ConfigList import ConfigListScreen

from enigma import eTimer
from Components.ProgressBar import ProgressBar 

from urlparse import urlparse, urlunparse

import xml.dom.minidom
import urllib
import urllib2
import os
import time
import hashlib

import httplib
from httplib import HTTP
from urlparse import urlparse
import urllib2

from twisted.web.client import downloadPage
from twisted.web.client import getPage

def convert_bytes(bytes):
    bytes = float(bytes)
    if bytes >= 1099511627776:
        terabytes = bytes / 1099511627776
        size = '%.2fT' % terabytes
    elif bytes >= 1073741824:
        gigabytes = bytes / 1073741824
        size = '%.2fG' % gigabytes
    elif bytes >= 1048576:
        megabytes = bytes / 1048576
        size = '%.2fM' % megabytes
    elif bytes >= 1024:
        kilobytes = bytes / 1024
        size = '%.2fK' % kilobytes
    else:
        size = '%.2fb' % bytes
    return size
	
	
def md5_for_file(file, block_size=2**20):

    f = open(file)

    md5 = hashlib.md5() 
    while True: 
        data = f.read(block_size) 
        if not data: 
            break 
        md5.update(data)

    f.close()
    return md5.hexdigest()

class Data:
	def __init__(self, val1=""):
		self.val1 = val1

class WidgetList(GUIComponent, object):
	def buildEntry(self, item):
		width = self.l.getItemSize().width()
		res = [ None ]
		
		res.append((eListboxPythonMultiContent.TYPE_TEXT, 0, 3, width, 30, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, _("%s ") % item.val1))

		return res

	def __init__(self):
		GUIComponent.__init__(self)
		self.l = eListboxPythonMultiContent()
		self.l.setFont(0, gFont("Regular", 18))
		self.l.setFont(1, gFont("Regular", 18))
		self.l.setBuildFunc(self.buildEntry)
		self.l.setItemHeight(30)
		self.onSelectionChanged = [ ]
		
	def connectSelChanged(self, fnc):
		if not fnc in self.onSelectionChanged:
			self.onSelectionChanged.append(fnc)

	def disconnectSelChanged(self, fnc):
		if fnc in self.onSelectionChanged:
			self.onSelectionChanged.remove(fnc)

	def selectionChanged(self):
		for x in self.onSelectionChanged:
			x()
	
	def getCurrent(self):
		cur = self.l.getCurrentSelection()
		return cur and cur[0]
	
	GUI_WIDGET = eListbox
	
	def postWidgetCreate(self, instance):
		instance.setContent(self.l)
		instance.selectionChanged.get().append(self.selectionChanged)

	def preWidgetRemove(self, instance):
		instance.setContent(None)
		instance.selectionChanged.get().remove(self.selectionChanged)

	def moveToIndex(self, index):
		self.instance.moveSelectionTo(index)

	def getCurrentIndex(self):
		return self.instance.getCurrentIndex()

	currentIndex = property(getCurrentIndex, moveToIndex)
	currentSelection = property(getCurrent)

	def setList(self, list):
		self.l.setList(list)
		
class EnigmaUpdate(Screen):
	skin = """
		<screen position="center,center" size="600,500" title="SoftwareUpdate" >
			<widget name="info" position="10,10" size="520,40" font="Regular;23" transparent="1" foregroundColor="#c2d2ff"/>
			<widget name="data" position="10,50" size="580,200" scrollbarMode="showOnDemand"/>
			<widget name="changelog_label" position="10,275" size="520,22" font="Regular;18" transparent="1" foregroundColor="#aec1f7"/>
			<widget name="changelog" position="10,300" font="Regular;16" transparent="1" size="540,135"/>
		    <widget name="progress_label" position="10,450" size="520,20" font="Regular;20" transparent="1" />
			<widget name="progress" position="10,475" size="580,20" borderWidth="1" />
		</screen>
		"""
	
	def __init__(self, session, args = None):
		self.skin = EnigmaUpdate.skin
		Screen.__init__(self, session)

		self["data"] = WidgetList()
		self.CurrentIndex=0
		
		counter=0
		filename = "/etc/image-version"
		for line in open(filename,'r'):
		
			line=line.strip()
			pieces=line.split("=")
			
			if(counter==0): self.version=pieces[1]
			if(counter==1): self.comment=pieces[1]
			if(counter==4): self.caturl=pieces[1]
			
			counter=counter+1
			
#		self.caturl="http://abipbox.com/enigmaupd/9900/e2/catalog.xml"
		#------------------------------	
		
		msg="Select version you want to install:"
		self["info"] = Label(msg)
		
		self["progress"] = ProgressBar()
		self["progress"].setValue(0)
		
		#casovac pre stiahnutie suboru
		self.selected_index=0
		self.downloading=False
		self.downloaded_data=0
		self.timeout = eTimer()
		self.timeout.callback.append(self.DownloadFile)
		
		#sem sa stiahne subor
		self.loc_file="/tmp/flash_img_kernel_root"
		
		#casovac pre spustenie flashtool
		self.timer = eTimer()
		self.timer.callback.append(self.flashtool)
		#-----------
		
		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
		{
		    "cancel": self.cancel,
		    "ok": self.ok,
		}, -2)
		
		t="Change log:"
		self["changelog_label"] = Label(t)
		
		self["changelog"] = Label("")

		t="Status: downloading data..."
		self["progress_label"] = Label(t)
		
		self.onLayoutFinish.append(self.layoutFinished)
	
	def DownloadFile(self):
		
		self.downloading=True
		
		chunk = 4096 
		data = self.urlfile.read(chunk) 
		if not data: 
		
			self.downloading=False
		
			self.fpath.close()
			self.urlfile.close()
			
			self.downloaded_data=0
			self.timeout.stop()
			
			#check md5
			md5_xml=self.images[self.selected_index]['md5']
			md5_local=md5_for_file(self.loc_file);
			
			if(md5_xml!=md5_local):
				self["progress_label"].setText("Status: error while downloading file. Incomplete file")
			else:
				# vsetko prebehlo OK, nasleduje instalacia
				self["progress_label"].setText("Checksum OK. Ready to upgrade.")
				self.session.openWithCallback(self.update,MessageBox,_("Are you sure you want to upgrade to this new version?\nYes - do you want to upgrade"), MessageBox.TYPE_YESNO)
			
		else:
			self.fpath.write(data)
			self.downloaded_data=self.downloaded_data+chunk
			
			percent=self.downloaded_data/self.urlfilesize*100
			percent=int(round(percent,0))
			
			size1=convert_bytes(self.downloaded_data)
			size2=convert_bytes(self.urlfilesize)
			
			msg="Status: downloading data: "+str(percent)+"%"+" ("+size1+"/"+size2+")"
			self["progress_label"].setText(msg)
			self["progress"].setValue(percent)
		

	def update(self,answer):
		if answer is True:
			self["progress_label"].setText("Please wait... do NOT switch off the receiver!")
			self.session.open(MessageBox,("Warning:\n%s") % ("Please wait... do NOT switch off the receiver!"),  MessageBox.TYPE_INFO)
			self.timer.start(15,True)
			pass
		else:
			self["progress_label"].setText("Status: ready")
			pass	
		pass

	def flashtool(self):
		os.system("/usr/bin/flashtool");
	
	def layoutFinished(self):	
		try:
			getPage(self.caturl).addCallback(self.ProcessXML).addErrback(self.error)
		except Exception, e:
			self["progress_label"].setText("Status: error while downloading data")

	def error(self, error):
		print self.caturl
		print error
		self.session.open(MessageBox,("UNEXPECTED ERROR:\n%s") % (error),  MessageBox.TYPE_INFO)
			
	def ProcessXML(self, html):

		#pouzi catalog.xml z internetu
		doc = xml.dom.minidom.parseString(html)
		
		# IBA NA TESTOVANIE!!!!!! inak zakomentovat!!!!
		#pouzi lokalny catalog.xml
		#doc = xml.dom.minidom.parse("/usr/local/lib/enigma2/python/Plugins/Extensions/EnigmaUpdate/catalog.xml")

		for node in doc.getElementsByTagName("changelog"):
			changelog_url = node.getAttribute("url")
			
		urlfilehandle = urllib.urlopen(changelog_url, proxies={})
		changelog_content=urlfilehandle.read()
		
		pieces=changelog_content.split("\n")
		self.changelog={}
		
		for line in pieces:
			line=line.strip()
			
			pieces2=line.split(" ")
			key=str(pieces2[0])
			
			msg=" ".join(pieces2[1:])
			
			if(key!=""):
				flag=self.changelog.has_key(key)
				if(flag==True):
					self.changelog[key].append(msg)
				else:
					self.changelog[key]=[]
					self.changelog[key].append(msg)
		
		l=[]
		
		self.images=[]
		
		#---pridame manual update, ak je subor v /tmp
		flag=os.path.exists(self.loc_file)
		if(flag==True):
			name="Manual upgrade"
			tmplist={"target": "", "name": name, "url": "local", "version": "", "creator": "", "md5": ""}
			self.images.append(tmplist)
			l.append(Data(name))

		for node in doc.getElementsByTagName("image"):
			target = node.getAttribute("target")
			target=target.encode('ascii')
			name = node.getAttribute("name")
			name=name.encode('ascii')
			url = node.getAttribute("url")
			url=url.encode('ascii')
			version = node.getAttribute("version")
			version=version.encode('ascii')
			creator = node.getAttribute("creator")
			creator=creator.encode('ascii')
			md5 = node.getAttribute("md5")
			md5=md5.encode('ascii')

			if(name!=""):
				tmplist={"target": target, "name": name, "url": url, "version": version, "creator": creator, "md5": md5}
				self.images.append(tmplist)
				l.append(Data(name))
			
		
		self["data"].connectSelChanged(self.onSelectionChanged)
		self["data"].setList([ (x,) for x in l])
		self["data"].show()
		
		self["progress_label"].setText("Status: ready")

	def onSelectionChanged(self):

		self.CurrentIndex=self["data"].instance.getCurrentIndex()
		
		url=self.images[self.CurrentIndex]['url']
		if(url=="local"):
			self["changelog"].setText("Manual install from HDD")
			return

		version=self.images[self.CurrentIndex]['version']
		key=version[4:]
		
		if(self.version==version):
			changelog_content="This is your current version\n"
		else: changelog_content=""
		
		flag=self.changelog.has_key(key)
		if(flag==True):
			changelog=self.changelog[key]
			changelog_content=changelog_content+"\n".join(changelog)
			self["changelog"].setText(changelog_content)
		else:
			self["changelog"].setText("")

		
	def ok(self):
	
		if(self.downloading==True): return
		self.selected_index=self.CurrentIndex
	
		#download file
		url=self.images[self.CurrentIndex]['url']
		
		#ak je v tmp, prejdeme priamo k install
		if(url=="local"):
			self.session.openWithCallback(self.update,MessageBox,_("Are you sure you want to upgrade to this new version?\nYes - do you want to upgrade"), MessageBox.TYPE_YESNO)
			return
		
		rem_file=url
		
		basename=os.path.basename(url)
		self.loc_file="/tmp/"+basename
		
		#subor kde sa to nakopiruje - nazov bude flash_img_kernel_root
		self.loc_file="/tmp/flash_img_kernel_root"

		try:
		
			self["progress"].setValue(0)
			self["progress_label"].setText("Status: downloading data...")
		
			#otvorime lokalny subor + url
			self.fpath = open(self.loc_file, "w")
			self.urlfile = urllib2.urlopen(rem_file)
			
			self.urlfilesize=int(self.urlfile.info()['Content-Length'])
			self.timeout.start(15,False)
			
		
		except IOError, e:
			print e
			self["progress_label"].setText("Status: I/O Error while downloading file.")
	
	def cancel(self):
		if(self.downloading==True):
			self.timeout.stop()
			self.fpath.close()
			os.remove(self.loc_file)
			
		self.close()



def main(session, **kwargs):
	session.open(EnigmaUpdate)

def Plugins(**kwargs):
	return [ PluginDescriptor(name="EnigmaUpdate", description="testing plugin", where = PluginDescriptor.WHERE_PLUGINMENU, fnc=main, icon="plugin.png") ]